package jadex.tools.generic;

import jadex.base.gui.SwingDefaultResultListener;
import jadex.base.gui.SwingDelegationResultListener;
import jadex.base.gui.componentviewer.IAbstractViewerPanel;
import jadex.commons.IPropertiesProvider;
import jadex.commons.Properties;
import jadex.commons.future.DelegationResultListener;
import jadex.commons.future.Future;
import jadex.commons.future.IFuture;
import jadex.commons.gui.JSplitPanel;
import jadex.commons.gui.ObjectCardLayout;
import jadex.commons.gui.SGUI;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.UIDefaults;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.basic.BasicComboBoxRenderer;


/**
 *  Panel that allows user choosing among different viewable objects (e.g. services or components).
 *  Uses a combobox for 
 */
public abstract class AbstractSelectorPanel extends JSplitPanel implements IPropertiesProvider
{
	//-------- constants --------

	/** The image icons. */
	protected static final UIDefaults icons = new UIDefaults(new Object[]
	{
		"viewer_empty", SGUI.makeIcon(AbstractGenericPlugin.class, "/jadex/tools/common/images/viewer_empty.png"),
	});
	
	/** The panel properties name. */
	public final String PANELPROPERTIES = "panelproperties";

	//-------- attributes --------
	
	/** The component selection box. */
	protected JComboBox selcb;
	
	/** The remote checkbox. */
	protected JCheckBox remotecb;
	
	/** The center panel. */
	protected JPanel centerp;
	
	/** The object card layout. */
	protected ObjectCardLayout ocl;
	
	/** The map of component viewer panels (cid->panel). */
	protected Map panels;
	
	/** The set properties. */
	protected Properties props;
	
	//-------- constructors --------
	
	/**
	 *  Create a new selector panel.
	 */
	public AbstractSelectorPanel()
	{
		this.setOrientation(VERTICAL_SPLIT);
		this.setOneTouchExpandable(true);
		this.setDividerLocation(0);
		this.setDividerLocation(0.0);

		panels = new HashMap();
		ocl = new ObjectCardLayout();
		centerp = new JPanel(ocl);
		
		JLabel emptylabel = new JLabel("Select instance that should be viewed",
		icons.getIcon("viewer_empty"), JLabel.CENTER);
		emptylabel.setVerticalAlignment(JLabel.CENTER);
		emptylabel.setHorizontalTextPosition(JLabel.CENTER);
		emptylabel.setVerticalTextPosition(JLabel.BOTTOM);
		emptylabel.setFont(emptylabel.getFont().deriveFont(emptylabel.getFont().getSize()*1.3f));
	
		centerp.add(ObjectCardLayout.DEFAULT_COMPONENT, emptylabel);
		
		selcb = new JComboBox(); 
		selcb.setRenderer(new BasicComboBoxRenderer()
		{
		    public Component getListCellRendererComponent(JList list, 
		    	Object value, int index, boolean isSelected, boolean cellHasFocus) 
		    {
		    	value	= value!=null? convertToString(value): null;
		    	return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		    }
		});
		
		remotecb = new JCheckBox("Remote");

		JPanel northp = new JPanel(new GridBagLayout());
		northp.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Instance Settings "));
		
		final JButton refreshb = new JButton("Refresh");
		int x=0;
		JLabel instl = new JLabel("Instance");
		instl.setToolTipText("Use the combo box to select the instance to be presented below.");
		selcb.setToolTipText("Use the combo box to select the instance to be presented below.");
		northp.add(instl, new GridBagConstraints(x++, 0, 1, 1, 0, 0, 
			GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,0,2), 0, 0));
		northp.add(selcb,  new GridBagConstraints(x++, 0, 1, 1, 1, 0, 
			GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0,2,0,2), 0, 0));
		northp.add(remotecb,  new GridBagConstraints(x++, 0, 1, 1, 0, 0, 
			GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,0,2), 0, 0));
		northp.add(refreshb,  new GridBagConstraints(x++, 0, 1, 1, 0, 0, 
			GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,0,2), 0, 0));
		
		refreshb.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent e) 
			{
				refreshCombo();
			}
		});
		selcb.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent e) 
			{
	//			System.out.println("Selected : "+selcb.getSelectedItem());
				
				final Object sel = selcb.getSelectedItem()!=null? selcb.getSelectedItem(): ObjectCardLayout.DEFAULT_COMPONENT;
				
				if(ocl.isAvailable(sel))
				{
					ocl.show(sel);
				}
				else
				{
					createPanel(sel).addResultListener(new SwingDefaultResultListener()
					{
						public void customResultAvailable(Object result)
						{
							final IAbstractViewerPanel panel = (IAbstractViewerPanel)result;
							IFuture	propsdone;
							if(props!=null && props.getSubproperty(PANELPROPERTIES)!=null)
							{
								propsdone	= panel.setProperties(props.getSubproperty(PANELPROPERTIES));
							}
							else
							{
								propsdone	= IFuture.DONE;
							}
							
							propsdone.addResultListener(new SwingDefaultResultListener()
							{
								public void customResultAvailable(Object result)
								{
									panels.put(sel, panel);
									centerp.add(panel.getComponent(), sel);
									ocl.show(sel);
								}
							});
						}
					});
				}
			}
		});

		this.setTopComponent(northp);
		this.setBottomComponent(centerp);
	}
	
	/**
	 *  Remove a panel.
	 */
	public void removePanel(Object key)
	{
		IAbstractViewerPanel panel = (IAbstractViewerPanel)panels.remove(key);
		if(panel!=null)
		{
			centerp.remove(panel.getComponent());
			panel.shutdown();
		}
	}
	
	/**
	 *  Refresh the combo box.
	 */
	public abstract void refreshCombo();
	
	/**
	 *  Create a panel for a component identifier.
	 */
	public abstract IFuture createPanel(Object element);

	/**
	 *  Convert object to string for property saving.
	 */
	public abstract String convertToString(Object element);
	
	
	
	/**
	 *  Get the selcb.
	 *  @return the selcb.
	 */
	public JComboBox getSelectionComboBox()
	{
		return selcb;
	}

	/**
	 *  Get the center panel.
	 *  @return the center panel.
	 */
	public JPanel getCenterPanel()
	{
		return centerp;
	}
	
	/**
	 *  Is remote.
	 *  @return True, if is remote.
	 */
	public boolean isRemote()
	{
		return remotecb.isSelected();
	}
	
	/**
	 *  Get the currently shown panel.
	 */
	public IAbstractViewerPanel getCurrentPanel()
	{
		IAbstractViewerPanel ret = null;
		Object key = ocl.getCurrentKey();
		if(key!=null)
		{
			ret = (IAbstractViewerPanel)panels.get(key);
		}
		return ret;
	}

	/**
	 *  Set properties loaded from project.
	 */
	public IFuture<Void> setProperties(final Properties props)
	{
		final Future<Void> ret = new Future<Void>();
		
		this.props = props;
		
//		System.out.println("set props: "+props);
		if(selcb.getSelectedItem()!=null && panels.containsKey(selcb.getSelectedItem()))
		{
			IAbstractViewerPanel panel = (IAbstractViewerPanel)panels.get(selcb.getSelectedItem());
			panel.setProperties(props.getSubproperty(PANELPROPERTIES))
				.addResultListener(new DelegationResultListener<Void>(ret));
		}
		else
		{
			ret.setResult(null);
		}
		
		return ret;
	}

	/**
	 *  Return properties to be saved in JCC settings.
	 */
	public IFuture<Properties> getProperties()
	{
		final Future<Properties> ret = new Future<Properties>();
		final Properties props = new Properties(null, getName(), null);
		if(selcb.getSelectedItem()!=null && panels.containsKey(selcb.getSelectedItem()))
		{
			IAbstractViewerPanel panel = (IAbstractViewerPanel)panels.get(selcb.getSelectedItem());
			panel.getProperties().addResultListener(new SwingDelegationResultListener<Properties>(ret)
			{
				public void customResultAvailable(Properties subprops) 
				{
//					Properties subprops = (Properties)result;
					props.addSubproperties(PANELPROPERTIES, subprops!=null ? subprops : new Properties());
					ret.setResult(props);
				};
			});
		}
		else
		{
			ret.setResult(null);
		}
		
//		System.out.println("props: "+props);
		return ret;
	}
	
	/** 
	 *  Shutdown the panel.
	 */
	public void shutdown()
	{
		for(Iterator it=panels.values().iterator(); it.hasNext(); )
		{
			// Todo: should wait for shutdown!!!
			((IAbstractViewerPanel)it.next()).shutdown();
		}
	}
}
